import os
from datetime import datetime
from typing import Optional
from uuid import UUID
from pydantic import BaseModel

from httpx import AsyncClient

from ..campaigns.models import ChannelType, CampaignGoal, Campaign as CampaignModel
from ..general.dataclasses import ResolvableCompanyModel
from ..integrations.dataclasses import LinkInfo


# Campaign object
class CampaignInfo(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    external_id: Optional[str]
    status: str
    last_sync_date: datetime
    ad_count: int
    channel: LinkInfo
    channel_type: ChannelType
    campaign_goal: Optional[CampaignGoal]

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "CampaignInfo | None":
        result = await ResolveClient.campaigns().get(f"campaigns/{object_id}/resolve", headers={"X-Organization-ID": str(organization_id)})

        if result.status_code != 200:
            return None

        return CampaignInfo(**result.json())

    @classmethod
    async def from_model(cls: "CampaignInfo", model: CampaignModel) -> 'CampaignInfo':
        channel = await LinkInfo.resolve_object(model.channel_id, model.organization)

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            external_id=model.external_id,
            status=model.status,
            last_sync_date=model.last_sync_date,
            ad_count=model.ad_count,
            channel=channel,
            channel_type=model.channel_type,
            campaign_goal=model.campaign_goal,
        )


class CampaignConfig(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    external_id: Optional[str]
    status: str
    last_sync_date: datetime
    ad_count: int
    channel_id: UUID
    channel_type: ChannelType
    campaign_goal: Optional[CampaignGoal]

    @classmethod
    async def from_model(cls: "CampaignConfig", model: CampaignModel) -> 'CampaignConfig':
        return cls(**model.model_dump())



from uuid import UUID

from sqlalchemy import Column, JSON
from sqlmodel import Field

from ..general.models import BaseModel


class IntegrationsModel(BaseModel):
    pass


class IntegrationsOrganizationModel(IntegrationsModel):
    organization: UUID = Field(index=True)



class Integration(IntegrationsModel, table=True):
    name: str = Field(index=True)
    icon: str = Field(index=True)
    type: str = Field(index=True)
    tag: str = Field(index=True, nullable=True)
    enabled: bool = Field(default=True, nullable=True)
    description: str = Field(index=True, nullable=True)
    code_reference: str = Field(index=True)
    setup_config: dict = Field(sa_column=Column(JSON))
    order: int = Field(default=0)


class Link(IntegrationsOrganizationModel, table=True):
    name: str = Field(index=True)
    status: str = Field(index=True)
    auth_config: dict = Field(sa_column=Column(JSON))
    integration_id: UUID = Field(foreign_key="integration.id")



import asyncio
import json
import os

import aio_pika

from ..event import Message, Queue, Event
from ..util.singleton import SingletonMeta


class RabbitMQ:
    def __init__(self):
        self.connection = None
        self.channel = None

    async def connect(self):
        self.connection = await aio_pika.connect_robust(
            host=os.getenv('QUEUE_HOST'),
            port=int(os.getenv('QUEUE_PORT')),
            login=os.getenv('QUEUE_USER'),
            password=os.getenv('QUEUE_PASS'),
        )
        print(f"  RabbitMQ - Connected!")
        self.channel = await self.connection.channel()
        print(f"  RabbitMQ - Channel created!")

    async def consume(self, queue: Queue, callback):
        print(f"  RabbitMQ - Consuming from queue: {queue}")
        queue_object = await self.channel.declare_queue(queue, durable=True)
        async for message in queue_object:
            async with message.process():
                event: Event = Event(**json.loads(message.body))
                print(f"  RabbitMQ ({queue}) - [x] Received event with type: {event.event_type}")
                await callback(queue, event)

    async def send_message(self, message: Message):
        await self.channel.default_exchange.publish(
            aio_pika.Message(body=message.event.encode()),
            routing_key=message.queue
        )
        print(f"  RabbitMQ ({message.queue.name}) - [x] Sent event with type: '{message.event.event_type}'")


class QueueContext(metaclass=SingletonMeta):

    def __init__(self):
        self._rabbit_mq = RabbitMQ()
        self._message_callbacks = []

    async def connect(self):
        await self._rabbit_mq.connect()

    async def start_consuming(self, queue: Queue):
        asyncio.create_task(
            self._rabbit_mq.consume(queue, self._on_new_message)
        )

    async def _on_new_message(self, queue: Queue, event: Event):
        for callback_object in self._message_callbacks:
            if callback_object.get("queue") == queue:
                await callback_object.get("callback")(event)

    def add_on_message_callback(self, queue: Queue, callback):
        self._message_callbacks.append({'queue': queue, 'callback': callback})

    async def send_message(self, message: Message):
        await self._rabbit_mq.send_message(message)


import enum

from uuid import UUID

from datetime import datetime
from enum import StrEnum
from typing import Optional

from sqlalchemy import Column, JSON, Enum
from sqlmodel import Field

from ..general.models import BaseModel


class ChannelType(StrEnum):
    JOB_BOARD = 'job-board'
    SOCIAL = 'social'


class CampaignGoal(enum.Enum):
    REACH = 'reach'
    TRAFFIC = 'traffic'
    CONVERSION = 'conversion'
    LEADS = 'leads'


class CampaignsModel(BaseModel):
    pass


class CampaignsOrganizationModel(CampaignsModel):
    organization: UUID = Field(index=True)


class Campaign(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    status: str = Field(index=True)
    last_sync_date: Optional[datetime] = Field()
    ad_count: int = Field(default=0)
    channel_id: UUID = Field(index=True)
    channel_type: ChannelType = Field(sa_column=Column(Enum(ChannelType)))
    campaign_goal: Optional[CampaignGoal] = Field(sa_column=Column(Enum(CampaignGoal)))


class VacancySelection(CampaignsOrganizationModel, table=True):
    campaign_id: UUID = Field(foreign_key="campaign.id")
    feed_id: UUID = Field(index=True)
    selection_criteria: dict = Field(sa_column=Column(JSON))


class AdSet(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    campaign_id: UUID = Field(foreign_key="campaign.id")
    platforms: list = Field(sa_column=Column(JSON))
    ad_types: list = Field(sa_column=Column(JSON))
    settings: dict = Field(sa_column=Column(JSON))


class TargetLocation(CampaignsOrganizationModel, table=True):
    ad_set: UUID = Field(foreign_key="adset.id")
    address: str = Field(index=True)
    distance: int = Field(index=True)


class TargetAudience(CampaignsOrganizationModel, table=True):
    ad_set: UUID = Field(foreign_key="adset.id")
    age_min: int = Field(index=True, default=18)
    age_max: int = Field(index=True, default=150)
    interests: list = Field(sa_column=Column(JSON))
    languages: list = Field(sa_column=Column(JSON))


class Ad(CampaignsOrganizationModel, table=True):
    name: str = Field(index=True)
    external_id: Optional[str] = Field(index=True)
    campaign_id: UUID = Field(foreign_key="campaign.id")
    ad_set_id: Optional[UUID] = Field(foreign_key="adset.id")
    vacancy_id: Optional[UUID] = Field()
    lead_form: Optional[UUID] = Field(foreign_key="leadform.id")
    primary_text: str = Field()
    title: str = Field()
    description: Optional[str] = Field()
    conversion_goal: Optional[str] = Field()


class LeadForm(CampaignsOrganizationModel, table=True):
    title: str = Field()



from uuid import UUID
from datetime import datetime
from typing import Optional
from pydantic import BaseModel

from .models import Link as LinkModel, Integration as IntegrationModel
from ..general.dataclasses import ResolvableModel, ResolvableCompanyModel
from talentro.services.clients import MSClient


# Integration object
class IntegrationInfo(ResolvableModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]

    name: str
    icon: str
    type: str
    tag: Optional[str]
    description: str

    @staticmethod
    async def resolve_object(object_id: UUID) -> "IntegrationInfo | None":
        result = await MSClient.integrations().get(f"integrations/{object_id}/resolve")

        if result.status_code != 200:
            return None

        return IntegrationInfo(**result.json())

    @classmethod
    async def from_model(cls: "IntegrationInfo", model: IntegrationModel) -> "IntegrationInfo":
        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            name=model.name,
            icon=model.icon,
            type=model.type,
            tag=model.tag,
            description=model.description,
        )


class IntegrationConfig(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]

    name: str
    icon: str
    type: str
    tag: str | None
    enabled: bool | None
    description: str | None
    code_reference: str
    setup_config: dict
    order: int

    @classmethod
    async def from_model(cls: "IntegrationConfig", model: IntegrationModel) -> "IntegrationConfig":
        return cls(**model.model_dump())


# Link object
class LinkInfo(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    integration: IntegrationInfo | None

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "LinkInfo | None":
        result = await MSClient.integrations().get(f"links/{object_id}/resolve", headers={"X-Organization-ID": str(organization_id)})

        if result.status_code != 200:
            return None

        return LinkInfo(**result.json())

    @classmethod
    async def from_model(cls: "LinkInfo", model: LinkModel) -> "LinkInfo | None":
        integration = await IntegrationInfo.resolve_object(model.integration_id)

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            status=model.status,
            integration=integration,
        )


class LinkConfig(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    auth_config: dict
    integration_id: UUID

    @classmethod
    async def from_model(cls: "LinkConfig", model: LinkModel) -> "LinkConfig":
        return cls(**model.model_dump())